#!/usr/bin/python
# Copyright 2016 CERN and GSI
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import difflib
import sys
import libxml2

from test.testBase import *
import genplcsrc
import iecommon
import iefiles

import libxml2

testFolder = "test"
generationFolder = testFolder + "/generated_temp"
comparisonFolder = testFolder + "/generated_correct"
generationFoldeController = generationFolder + "/controller"
comparisonFolderController = comparisonFolder + "/controller"
def generatePLCSources():
    deployDOM = libxml2.parseFile(testFolder + "/AllTypesDU.silecsdeploy")
    controllerNodes = deployDOM.xpathEval("/SILECS-Deploy/Controller")
    for controllerNode in controllerNodes:
        paramsFile = generationFolder + "/client/" + controllerNode.prop("host-name") + ".silecsparam"
        genplcsrc.generateControllerCode(controllerNode, paramsFile, generationFoldeController ,{'errorlog': True})
    deployDOM.freeDoc()

def CompareGeneratedFiles(hostName,fileExtension):
    fileGeneratedPath = generationFoldeController + "/" + hostName + fileExtension
    fileCorrectPath = comparisonFolderController + "/" + hostName + fileExtension
    print "comparing: " + fileGeneratedPath + " with: " +  fileCorrectPath
    assertPLCCodeEqual( fileGeneratedPath, fileCorrectPath)

def SiemensSourcesTest():
    CompareGeneratedFiles("Siemens_TiaDevice",".scl")
    CompareGeneratedFiles("Siemens_TiaDevice",".sdf")
    CompareGeneratedFiles("Siemens_TiaBlock",".scl")
    CompareGeneratedFiles("Siemens_TiaBlock",".sdf")
    CompareGeneratedFiles("Siemens_Step7Device",".scl")
    CompareGeneratedFiles("Siemens_Step7Device",".sdf")
    CompareGeneratedFiles("Siemens_Step7Block",".scl")
    CompareGeneratedFiles("Siemens_Step7Block",".sdf")

def BeckhoffSourcesTest():
    CompareGeneratedFiles("Beckhoff_BC9020",".exp")
    CompareGeneratedFiles("Beckhoff_CX9020",".exp")

def SchneiderSourcesTest():
    CompareGeneratedFiles("Schneider_M340",".xsy")
    CompareGeneratedFiles("Schneider_PremiumQuantum",".xsy")

def RabbitSourcesTest():
    CompareGeneratedFiles("Rabbit_BlockMode",".h")
    CompareGeneratedFiles("Rabbit_DeviceMode",".h")

def runTests():

    generatePLCSources()

    SiemensSourcesTest()
    BeckhoffSourcesTest()
    SchneiderSourcesTest()
    RabbitSourcesTest()
    allTestsOk()
