// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _SILECS_PLC_ACTION_H_
#define _SILECS_PLC_ACTION_H_

#include <string>
#include <silecs-communication/interface/utility/Mutex.h>
#include <silecs-communication/interface/core/SilecsAction.h>

namespace Silecs
{
	class Block;
	class Context;

	/*!
	 * \class PLCAction
	 * \brief PLCAction defines the abstract level of any PLC actions (recv and send)
	 */
	class PLCAction: public Action
	{

	public:
		PLCAction(Block* block) : Action(block){
	        bufferMux_  = new Mutex("bufferMux");
		};
 		virtual ~PLCAction() {
 	        delete bufferMux_;
 		};

        /*!
         * \fn execute
         * It contains the code to be executed for this action.
         * Overwrites the code from Action class
         */
		virtual int execute(Context* pContext) = 0;

	protected:
        //The frame buffer update (import/export registers) and related read/write data must be done
        //in a critical section (the same buffer is used to send/recv data to all devices).
        inline void bufferLock()      { bufferMux_->lock();   }
        inline void bufferUnlock()    { bufferMux_->unlock(); }

	private:
        Mutex* bufferMux_;

	};

} // namespace

#endif // _SILECS_PLC_ACTION_H_
