// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _SILECS_PLC_BLOCK_H_
#define _SILECS_PLC_BLOCK_H_

#include <silecs-communication/interface/core/SilecsService.h>
#include <silecs-communication/interface/utility/Thread.h>
#include <silecs-communication/interface/equipment/SilecsBlock.h>

namespace Silecs
{

	class PLCBlock : public Block
	{

	public:
		PLCBlock(PLC* thePLC,ElementXML blockNode, AccessType accessType);
		~PLCBlock();

		inline unsigned long& getBufferSize() { return bufferSize_; }

	protected:
		friend class Device;

		/// Send/Receive data size (size of block * nb of device)
		unsigned long bufferSize_;

	};

	/*! -----------------------------------------------------------------------
	 * \class InputBlock
	 * \brief This is a specific Block for Acquisition data (from PLC to client)
	 */
	class InputBlock : public PLCBlock
	{

	public:
		InputBlock(PLC* thePLC,ElementXML blockNode, AccessType accessType);
		virtual ~InputBlock();

	};

	/*! -----------------------------------------------------------------------
	 * \class OutputBlock
	 * \brief This is a specific Block for Settings (from client to PLC)
	 */
	class OutputBlock : public PLCBlock
	{

	public:
		OutputBlock(PLC* thePLC,ElementXML blockNode, AccessType accessType);
		virtual ~OutputBlock();

	};

} // namespace

#endif // _SILECS_BLOCK_H_
