// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "StringUtilities.h"
#include <stdio.h>


namespace Silecs
{
	const unsigned int StringUtilities::c_emptyStringArraySize = 1;
	const char StringUtilities::c_emptyString[] = "\0";
	const char* StringUtilities::c_emptyStringArray[1] = {"\0"};

	void StringUtilities::tokenize(const std::string& str, std::vector<std::string>& tokens,
			const std::string& delimiters) {

	  size_t startPos = 0;
	  size_t endPos = str.find_first_of(delimiters, 0);
	  if (endPos == std::string::npos) {
		//no delimiters
		tokens.push_back(str);
		return;
	  }
	  size_t maxLenght = str.size();
	  while (startPos != std::string::npos && endPos != std::string::npos) {
		// Found a token, add it to the vector.
		std::string s = str.substr(startPos, endPos - startPos);
		tokens.push_back(s);
		// Next start delimiter
		if (endPos < maxLenght -1)
		  startPos = endPos+1;
		else
		  // string size exceeded
		  break;
		endPos = str.find_first_of(delimiters, startPos);
		if (endPos == std::string::npos) {
		  // no delimiter at the end : last token
		  std::string s = str.substr(startPos, maxLenght - startPos);
		  tokens.push_back(s);
		}
	  }
	}

	void StringUtilities::trimWhiteSpace(std::string& str) {
		size_t pos = str.find(" ");
		while (pos != std::string::npos) {
			str.erase(pos, 1);
			pos = str.find(" ");
		}
	}

	std::string StringUtilities::toString(unsigned int data)
	{
		char tmp[32];
		sprintf(tmp, "%u", data);
		return tmp;
	}

	std::string StringUtilities::toString(int data)
	{
		char tmp[32];
		sprintf(tmp, "%d", data);
		return tmp;
	}

	std::string StringUtilities::toString(long data)
	{
		char tmp[32];
		sprintf(tmp, "%ld", data);
		return tmp;
	}

	std::string StringUtilities::toString(unsigned long data)
	{
		char tmp[32];
		sprintf(tmp, "%lu", data);
		return tmp;
	}

	std::string StringUtilities::toString(long long data)
	{
		char tmp[32];
		sprintf(tmp, "%lld", data);
		return tmp;
	}

	std::string StringUtilities::toString(double data)
	{
		char tmp[32];
		sprintf(tmp, "%f", data);
		return tmp;
	}

	std::string StringUtilities::toString(const void * ptr)
	{
		char tmp[32];
		sprintf(tmp, "%p", ptr);
		return tmp;
	}

	void StringUtilities::toLower(std::string& str)
	{  //cannot used std::transform() method because of ppc4 support!
	   for(unsigned int i=0;i<str.length();i++)
	   { str[i] = tolower(str[i]);
	   }
	}

} // namespace
