// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _STRING_UTILITIES_H_
#define _STRING_UTILITIES_H_

#include <string>
#include <vector>
#include <silecs-communication/interface/utility/SilecsException.h>
#include <sstream>

namespace Silecs
{

	class StringUtilities
	{
		public:
			static void tokenize(const std::string& str, std::vector<std::string>& tokens,
									const std::string& delimiters = " ");

			static void trimWhiteSpace(std::string& str);

			// define an array of one pointer on an empty string
			static const unsigned int c_emptyStringArraySize;
			static const char* c_emptyStringArray[1];
			// define an empty string
			static const char c_emptyString[];

			static std::string toString(unsigned int data);
			static std::string toString(int data);
			static std::string toString(unsigned long data);
			static std::string toString(long data);
			static std::string toString(long long data);
			static std::string toString(double data);
			static std::string toString(const void * ptr);
			static void toLower(std::string& str);

		    /*!
		     * \brief transform a string into any integer-type
		     * \param str the string to transform
		     * \param value that will receive the conversion
		     */
		    template<typename T> static void fromString(T& value, const std::string& str);

	};

	template<typename T>
	inline void StringUtilities::fromString(T& value, const std::string& str)
	{
	    std::istringstream iss(str);
	    if ((iss >> value).fail())
	    {
			std::ostringstream message;
			message << "Failed to convert the string: '" << str << "' to numeric type";
			throw SilecsException(__FILE__, __LINE__,message.str().c_str());
	    }
	}

} // namespace


#endif //_STRING_UTILITIES_H_
