// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
 *
 * Revision : v1.0
 * Date     : 01/2010 - F.Locci
 * Objet    : Initial version - From previous Silecs-2
 * Action   :
 */

#ifndef _SILECS_H_
#define _SILECS_H_

/*
 * This library was developped to provide communication between
 * Linux/LynxOS active host (client) and a PLC passive server.
 * Based on S7 or MODBUS std. protocole, it defines generic
 * set and get functions to read and write data into/from PLC
 * memory.
 *
 */

#ifdef __cplusplus
extern "C" {
#endif

/* ---------------------------------------------------------*/
/* INCLUDE FILES																						*/
/* ---------------------------------------------------------*/
#include "ietype.h"

/* ---------------------------------------------------------*/
/* PROTOTYPE DEFINITIONS                                    */
/* ---------------------------------------------------------*/
#include <silecs-communication/protocol/modbus/iemdb.h>  /* modbus interface  */

/*----------------------------------------------------------*/
/* Time funtion
 * IeRfcSetTime: Convert time_t epoch date to PLC _DT format
 * (SIMATIC).
 *
 * Details:
 * PLC DATE_AND_TIME (or DT) format is coded on 8 byte in BCD
 * format:
 * 0      1      2      3      4      5      6       7
 * YY     MM     DD     HH     MN     SC     SC/100  SC/10e4
 */
void IeRfcSetTime(unsigned char *dt, time_t epoch);  // to PLC

/*----------------------------------------------------------*/
/* Time funtion
 * IeRfcGetTime: Convert PLC _DT format to lynxOS time_t format
 * (SIMATIC).
 *
 * Details:
 * PLC DATE_AND_TIME (or DT) format is coded on 8 byte in BCD
 * format:
 * 0      1      2      3      4      5      6       7
 * YY     MM     DD     HH     MN     SC     SC/100  SC/10e4
 */
double IeRfcGetTime(unsigned char *dt); // from PLC

/*----------------------------------------------------------*/
/* IeRfcPing function is a function used to check
 * if PLC is ON (~ping) before trying to connect it.
 */
int IeRfcPing (char *hostName, char *plcIP);

/*----------------------------------------------------------*/
/* This function return the string which describes the
 * error given by 'error' parameter.
 * 'error': error from ie library or PLC server (in)
 * return string pointer
 *
 * Details:
 */
char *IeGetErrorMessage(int err);

/*----------------------------------------------------------*/
/* This function initializes the DSC-PLC communication.
 * return 0 or <0 on error (see constant error)
 *
 * Details:
 * Must be executed at first.
 */
int IeInit (void);

/*----------------------------------------------------------*/
/* This function initializes the DSC-PLC communication.
 * return 0 or <0 on error (see constant error)
 *
 * Details:
 * Must be executed at end.
 */
int IeExit (void);

#ifdef __cplusplus
}
#endif

#endif /* _SILECS_H_ */
