/* 		modbus_rtu.h

   By P.Costigan email: phil@pcscada.com.au http://pcscada.com.au

   These library of functions are designed to enable a program send and
   receive data from a device that communicates using the Modbus protocol.

   Copyright (C) 2000 Philip Costigan  P.C. SCADA LINK PTY. LTD.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.




   The functions included here have been derived from the
   Modicon Modbus Protocol Reference Guide
   which can be obtained from Schneider at www.schneiderautomation.com.

   This code has its origins with
   paul@pmcrae.freeserve.co.uk (http://www.pmcrae.freeserve.co.uk)
   who wrote a small program to read 100 registers from a modbus slave.

   I have used his code as a catalist to produce this more functional set
   of functions. Thanks paul.


 */


#ifndef MODBUS_MASTER_H
#define MODBUS_MASTER_H

#ifdef __cplusplus
extern "C" {
#endif

/*F.Locci #include <plc.h>*/  /* get the plc data types */
#include <time.h> /* struct timespec data structure */

#include "mb_addr.h"  /* get definition of common variable types */



/***********************************************************************

	 Note: All functions used for sending or receiving data via
	       modbus return these return values.


	Returns:	string_length if OK
			0 if failed
			Less than 0 for exception errors

***********************************************************************/


/*F.Locci: error constants adjustement to be SILECS conformed
#define COMMS_FAILURE 0
#define ILLEGAL_FUNCTION -1
#define ILLEGAL_DATA_ADDRESS -2
#define ILLEGAL_DATA_VALUE -3
#define SLAVE_DEVICE_FAILURE -4
#define ACKNOWLEDGE -5
#define SLAVE_DEVICE_BUSY -6
#define NEGATIVE_ACKNOWLEDGE -7
#define MEMORY_PARITY_ERROR -8

#define PORT_FAILURE -11
*/
#define PORT_FAILURE -3


/************************************************************************

	read_bdata

    F.Locci: just read raw byte array. Take care of frame longer than 
	125 words.

*************************************************************************/

#define MAX_READ_BDATA  200

int read_bdata(int slave,    /* slave id */
             int start_addr, /* word pointer addr. */
             int count,      /* byte count to read */
             u8 *dest,       /* byte array */
             int ttyfd,      /* connection id */
             int send_retries, /* number of request retries */
             const struct timespec *response_timeout); /* timeout value */


/*************************************************************************

	write_bdata

    F.Locci: just write raw byte array. Take care of frame which exceeds 
    modbus frame std. length.
	!!byte count MUST be even value - 16 bits alignment in that PLC

***************************************************************************/

#define MAX_WRITE_BDATA  200

int write_bdata(int slave,   /* slave id */
             int start_addr, /* word pointer addr.  */
             int count,      /* byte count to write - MUST be even value */
             u8 *data,       /* byte array */
             int ttyfd,      /* connection id */
             int send_retries, /* number of request retries */
             const struct timespec *response_timeout); /* timeout value */


/************************************************************************

	initialise

	This function sets up the libraries
        (allocate memory for buffers, initialise data strcutures, etc)

**************************************************************************/
int mb_master_init(int nd_count);

int mb_master_done(void);


/***************************************************************************

	set_up_comms

	This function sets up (closes) a connection to a remote modbus
	slave.


***************************************************************************/
int mb_master_connect(node_addr_t node_addr);

int mb_master_close(int nd);

#ifdef __cplusplus
}
#endif

#endif  /* MODBUS_MASTER_H */

