// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <silecs-communication/wrapper/DeployUnit.h>

namespace SilecsWrapper
{

DeployConfig::DeployConfig() :
                _automaticConnect(true)
{
}

DeployConfig::DeployConfig(bool automaticConnect) :
                _automaticConnect(automaticConnect)
{
}

bool DeployConfig::getAutomaticConnect() const
{
    return _automaticConnect;
}

void DeployConfig::setAutomaticConnect(bool automaticConnect)
{
    _automaticConnect = automaticConnect;
}

//===============================================

DeployUnit* DeployUnit::_instance = NULL;
//SilecsWrapper::DeployConfig DeployUnit::_globalConfig = SilecsWrapper::DeployConfig();

DeployUnit::DeployUnit(const std::string& name, const std::string& version, const std::string& logTopics,const DeployConfig& globalConfig) :
                _name(name),
                _version(version),
                _globalConfig(globalConfig)
{
    // Process log topics
    char *argv[2] =
    { (char *) "-plcLog", const_cast<char*>(logTopics.c_str()) };

    // Get singleton instance
    _silecsService = Silecs::Service::getInstance(2, argv);
}

void DeployUnit::deleteInstance()
{
    if (_instance != NULL)
    {
        delete _instance;
        _instance = NULL;
    }
}

const std::string& DeployUnit::getName() const
{
    return _name;
}

const std::string& DeployUnit::getVersion() const
{
    return _version;
}

Silecs::Service* DeployUnit::getService() const
{
    return _silecsService;
}

DeployConfig& DeployUnit::getGlobalConfig()
{
    return _globalConfig;
}

void DeployUnit::setGlobalConfig(DeployConfig& globalConfig)
{
    _globalConfig = globalConfig;
}

DeployUnit::~DeployUnit()
{
}

}
 //namespace SilecsWrapper
