// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SILECSWRAPER_DEPLOY_UNIT_H_
#define SILECSWRAPER_DEPLOY_UNIT_H_

#include <silecs-communication/interface/core/SilecsService.h>

#include <string>

namespace SilecsWrapper
{

    class DeployConfig
    {
    public:
        DeployConfig();
        DeployConfig(bool automaticConnect);
        bool getAutomaticConnect() const;
        void setAutomaticConnect(bool automaticConnect);

    private:
        bool _automaticConnect;

    };

    class DeployUnit
    {
    public:
    DeployUnit(const std::string& name, const std::string& version, const std::string& logTopics,
            const DeployConfig& globalConfig);
        virtual ~DeployUnit();

        void deleteInstance();

        const std::string& getName() const;
        const std::string& getVersion() const;
        Silecs::Service* getService() const;
    DeployConfig& getGlobalConfig();

        void setGlobalConfig(DeployConfig& globalConfig);

    protected:
        const std::string _name;
        const std::string _version;
    DeployConfig _globalConfig;
        Silecs::Service* _silecsService;
        //singleton instance
        static DeployUnit* _instance;

    private:
        // Non copyable
        DeployUnit(const DeployUnit& other);
        DeployUnit& operator=(const DeployUnit&);

    };

} //namespace SilecsWrapper

#endif
