// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SILECSWRAPER_DESIGN_H_
#define SILECSWRAPER_DESIGN_H_

#include <silecs-communication/interface/equipment/SilecsCluster.h>
#include <silecs-communication/wrapper/DeployUnit.h>

#include <string>

namespace SilecsWrapper
{
class DeployUnit;
class Controller;

typedef Silecs::ClusterConfig DesignConfig;

class Design
{
public:
    Design(const std::string& name, const std::string& version, DeployUnit *deployUnit);

    const std::string& getName() const;

    const std::string& getVersion() const;

    Silecs::Cluster* getSilecsCluster() const;

    DeployUnit* getDeployUnit() const;

    void setConfiguration(const DesignConfig& configuration);

    const DesignConfig& getConfiguration();

    virtual ~Design();

protected:
    const std::string _name;
    const std::string _version;
    DeployUnit* _deployUnit;
    Silecs::Cluster* _silecsCluster;
    const DeployConfig _config;

private:
    // Non copyable
    Design(const Design& other);
    Design& operator=(const Design&);
};

} //namespace SilecsWrapper

#endif
