// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef SILECSWRAPER_DEVICE_H_
#define SILECSWRAPER_DEVICE_H_

#include <silecs-communication/interface/core/SilecsService.h>
#include <silecs-communication/interface/equipment/SilecsDevice.h>
#include <silecs-communication/wrapper/Controller.h>

#include <string>

namespace SilecsWrapper
{

class Device
{
public:
    Device(const std::string& label, Controller *controller) :
                    _label(label),
                    _controller(controller)
    {
        _silecsDevice = _controller->getSilecsPLC()->getDevice(label);
    }

    const std::string& getLabel() const
    {
        return _label;
    }

    Silecs::Device* getSilecsDevice()
    {
        return _silecsDevice;
    }

    virtual ~Device()
    {
    }

protected:
    std::string _label;
    Controller* _controller;
    Silecs::Device* _silecsDevice;

private:
    // Non copyable
    Device(const Device& other);
    Device& operator=(const Device&);
};

} //namespace SilecsWrapper

#endif
