// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <silecs-diagnostic/utils.h>
#include <silecs-diagnostic/silecsmodule.h>

extern silecsModule *mysilecs;

void Utils::logError(QTextBrowser* errorConsole, std::string message)
{
	errorConsole->insertHtml(QString::fromStdString("<font color='red'>DIAG-TOOL [ERROR] " + message + "</font><br>"));
	errorConsole->verticalScrollBar()->setValue(errorConsole->verticalScrollBar()->maximum());
}

void Utils::logInfo(QTextBrowser* errorConsole, std::string message)
{
	errorConsole->insertHtml(QString::fromStdString("<font color='black'>DIAG-TOOL [INFO] " + message + "</font><br>"));
	errorConsole->verticalScrollBar()->setValue(errorConsole->verticalScrollBar()->maximum());
}

void Utils::logDebugIf(QTextBrowser* errorConsole, std::string message)
{
	if( mysilecs->isDebugLogEnabled())
	{
		errorConsole->insertHtml(QString::fromStdString("<font color='black'>DIAG-TOOL [DEBUG] " + message + "</font><br>"));
		errorConsole->verticalScrollBar()->setValue(errorConsole->verticalScrollBar()->maximum());
	}
}

//convert integer to string
std::string Utils::convertInt(int number)
{
    char tmp[32];
    sprintf(tmp, "%d", number);
    return tmp;
}

//convert long to string
std::string Utils::convertLong(long number)
{
    char tmp[32];
    sprintf(tmp, "%ld", number);
    return tmp;
}

Item* Utils::addTreeItem(Item *parent,QString name ,QString description, QString type, void* linkedObject,QString icon)
{
    Item *itm = new Item(linkedObject);
    itm->setFlags(Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled );
    itm->setText(0,name);
    itm->setText(2,description);
    itm->setWhatsThis(0,type);

    if(icon.compare("")!=0)
        itm->setIcon(0,QIcon(QPixmap(icon)));

    parent->addChild(itm);
    return itm;
}

char* Utils::chop(char *string)
{
    size_t i, len;
    len = strlen(string);
    char *newstring;
    newstring = (char *)malloc(len);
    for(i = 0; i < strlen(string)-1; i++)
    {
        newstring[i] = string[i];
    }
    newstring[len] = '\0';

    return newstring;
}

template <class T>bool from_string(T& t,const std::string& s, std::ios_base& (*f)(std::ios_base&))
{
    std::istringstream iss(s);
    return !(iss >> f >> t).fail();
}

void Utils::displayClusterInformation(Silecs::Cluster *cluster,QTextEdit *console)
{
    std::string text = "";

    text.append("<h3>Cluster general information</h3>");
    text.append("<ul>");
    text.append("<li>Class name:    "+cluster->getClassName()+"</li>");
    text.append("<li>Class version: "+cluster->getClassVersion()+"</li>");
    text.append("<li>Host name:     "+cluster->getHostName()+"</li>");
    text.append("</ul>");

    console->setText(QString::fromStdString(text));

}

void Utils::displayPLCInformation(Silecs::PLC *plc,QTextEdit *console)
{
    std::string text = "";

    //TODO: GSI Add Header Info
    if(plc->isConnected())
    {
//        text.append("<h3><font color ='green'>This PLC is currently connected</font></h3><hr/>");
//
//        HeaderInformations *headerInfo = mysilecs->getHeaderInformations(plc->getName(),plc->getParamsFileName());
//        text.append("<h3><font color ='green'>PLC Runtime information</font></h3>");
//        text.append("<ul>");
//        text.append("<li>PLC HEADER DATA uploaded from: "+plc->getName()+"</li>");
//        text.append("<li>Software Release: "+headerInfo->getheaderRelease()+"</li>");
//        text.append("<li>Mapping Owner:    "+headerInfo->getheaderOwner()+"</li>");
//        text.append("<li>Mapping Checksum: "+headerInfo->getHeaderChecksum()+"</li>");
//        text.append("<li>Generation Date:  "+headerInfo->getheaderDate()+"</li>");
//        text.append("</ul><hr/>");
//
//        delete headerInfo;
    }
    else
        text.append("<h3>This PLC is currently disconnected</h3><hr/>");

    text.append("<h3>PLC general information</h3>");
    text.append("<ul>");
    text.append("<li>Name:    "+plc->getName()+"</li>");
    text.append("<li>Address: "+plc->getIPAddress()+"</li>");
    text.append("<li>Brand:   "+plc->getBrand()+"</li>");
    text.append("<li>Model:   "+plc->getModel()+"</li>");
    text.append("<li>Owner:   "+plc->getLocalOwner()+"</li>");
    text.append("</ul><hr/>");

    text.append("<h3>PLC configuration information</h3>");
    text.append("<ul>");
    text.append("<li>Protocol type: "+plc->getProtocolType()+"</li>");
    text.append("<li>Protocol mode: "+plc->getProtocolMode()+"</li>");
    text.append("<li>Base address:  "+convertLong(plc->getBaseAddress())+"</li>");
    text.append("</ul><hr/>");

    text.append("<h3>PLC generation information</h3>");
    text.append("<ul>");
    text.append("<li>Release:  "+plc->getLocalRelease()+"</li>");
    text.append("<li>Date:     "+plc->getLocalDate()+"</li>");
    text.append("<li>Checksum: "+convertLong(plc->getLocalChecksum())+"</li>");
    text.append("</ul><hr/>");

    console->setText(QString::fromStdString(text));
}

void Utils::displayDeviceInformation(Silecs::Device *device,QTextEdit *console)
{
    std::string text = "";

    text.append("<h3>Device general information</h3>");
    text.append("<ul>");
    text.append("<li>Name:    "+device->getLabel()+"</li>");
    text.append("</ul>");

    console->setText(QString::fromStdString(text));
}

void Utils::displayRegisterInformation(Silecs::Register *reg,QTextEdit *console)
{
    std::string text = "";

    text.append("<h3>Register general information</h3>");
    text.append("<ul>");
    text.append("<li>Name:        "+reg->getName()+"</li>");
    text.append("<li>Format:      "+reg->getFormatAsString()+"</li>");
    if(reg->getFormat() == String)
        text.append("<li>String length: "+convertLong(reg->getLength())+"</li>");
    text.append("<li>Dimension1:   "+convertLong(reg->getDimension1())+"</li>");
    text.append("<li>Dimension2:  "+convertLong(reg->getDimension2())+"</li>");
    text.append("<li>Block name:  "+reg->getBlockName()+"</li>");
    text.append("<li>Last update: "+reg->getTimeStampAsString()+"</li>");

    // Access method
    if(reg->hasInputAccess() && reg->hasOutputAccess())
        text.append("<li>Access method: Read / Write </li>");
    else
    {
        if(reg->hasInputAccess()) text.append("<li>Access method: Read only </li>");
        else text.append("<li>Access method: Write only </li>");
    }

    text.append("<li>Synchronization method:  "+reg->getSynchroTypeAsString()+"</li>");

    // Retentive volatile
    if(reg->isRetentive())
        text.append("<li>Storage method: Retentive</li>");
    else
        text.append("<li>Storage method: Volatile</li>");

    text.append("</ul>");

    console->setText(QString::fromStdString(text));

}

void Utils::displayRegisterValue(Silecs::Register *reg,
                                 QLabel *binValueLabel,
                                 QLabel *hexValueLabel,
                                 QLabel *decValueLabel,
                                 QLabel *asciiValueLabel
                                 )
{

    if(!reg->hasInputAccess()){
        binValueLabel->setText("--Write only register--");
        hexValueLabel->setText("--Write only register--");
        decValueLabel->setText("--Write only register--");
        asciiValueLabel->setText("--Write only register--");
    }
    else
    {
        // The register has input access
        if(reg->getDimension1()>1 || reg->getDimension2()>1)
        {
            binValueLabel->setText("--Vector--");
            hexValueLabel->setText("--Vector--");
            decValueLabel->setText("--Vector--");
            asciiValueLabel->setText("--Vector--");
        }
        else{
            //Scalar with input access
            switch(reg->getFormat())
            {
                case uInt8:
                {

                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt8(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt8(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt8(), 0, 10));
                    //QString temp;
                    //asciiValueLabel->setText(temp.sprintf("%c",reg->getValUInt8()));
                    QChar c = reg->getValUInt8();
                    //if(!c.isPrint())
                    //    c='?';
                    //asciiValueLabel->setText(QObject::tr("%1").arg(c));
                    asciiValueLabel->setText(c.isPrint() ? QObject::tr("%1").arg(c):"--Not printable--");

                    break;
                }
                case Int8:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValInt8(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValInt8(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValInt8(), 0, 10));
                    QChar c = reg->getValInt8();
                    asciiValueLabel->setText(c.isPrint() ? QObject::tr("%1").arg(c):"--Not printable--");
                    break;
                }
                case uInt16:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt16(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt16(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt16(), 0, 10));
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                case Int16:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValInt16(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValInt16(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValInt16(), 0, 10));
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                case uInt32:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt32(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt32(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt32(), 0, 10));
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                case Int32:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValInt32(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValInt32(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValInt32(), 0, 10));
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                case uInt64:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt64(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt64(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValUInt64(), 0, 10));
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                case Int64:
                {
                    binValueLabel->setText(QObject::tr("%1").arg(reg->getValInt64(), 0, 2));
                    hexValueLabel->setText(QObject::tr("%1").arg(reg->getValInt64(), 0, 16));
                    decValueLabel->setText(QObject::tr("%1").arg(reg->getValInt64(), 0, 10));
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                case String:
                {
                    binValueLabel->setText("--Not relevant--");
                    hexValueLabel->setText("--Not relevant--");
                    decValueLabel->setText("--Not relevant--");
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }

                default:// Float32,Float64,Date
                {
                    binValueLabel->setText("--Not relevant--");
                    hexValueLabel->setText("--Not relevant--");
                    decValueLabel->setText("--Not relevant--");
                    asciiValueLabel->setText("--Not relevant--");
                    break;
                }
            }
        }
    }
}
